%
% run_pantilt_sim.m
%
% Run a simulation of the pan-tilt camera robot, brains and all.
%
% Last modification: 8/30/07


% Clear all variables and close all windows.
% clear all;
% close all;

% Load simulation run parameters.
load sim_run_params;

% Load simulation script arguments.
load sim_run_sargs;

% Initialize the simulator types.
init_PISHTAR_types;

% Initialize the simulator.
init_simrun;

% If we're running the brain model, do the appropriate initialization.
if (brain_type == brain_cog_model)
    init_cog_model_brain;
end;

% If we are to display the scene, create the figure window.
if (display_scene == 1)
    make_disp_fig;
end;

%
% Simulation Loop
%
%   Overall Flow:
%      Read and process keystrokes.
%      Update the simulation state.
%         Check to see if event times out, and deal with event termination.
%         If in a null event, initiate a new event.
%         Terminate any expired reward.
%         Terminate the possibility of a reinforcement if the post-stimulus
%             delivery window has expired.
%         Terminate any expired punisher.
%         Detect delayed reward and deliver if appropriate.
%         Detect delayed punisher and deliver if appropriate.
%         Detect and respond to Nod.
%         Detect and respond to Shake.
%         Update the reward timer.
%         Update the post-stimulus reinforcement delivery window timer.
%         Update the stimulus delayed reward timer.
%         Update the punisher timer.
%         Update the stimulus delayed punisher timer.
%         Update the event timer.
%         Update the position of the visual stimulus.
%         Update the scene viewable by the camera (and brain).
%      Update the body (camera) state (the camera's position in the scene).
%      Update the brain state.
%         Read color and B/W info from the camera.
%         Update working memory for color and tones.
%         Update novelty signal for color and tone stimuli.
%         Update neural units OR run rule-based algorithm.
%         Implement the tracking behavior or Nod or Shake.
%         Update the camera speed according to brain's choice.
%      Render the scene for display.
%

% Loop over all iterations...
for iter = 1:num_iters
    
% Print out strings telling how many iterations have passed.
if (mod(iter,monitor_interval) == 0)
    disp(['Iteration ',num2str(iter),' (',num2str(iter * iter_duration), ...
        ' ms)']);
end;

%
% Read and process keystrokes.
%
if (get_key_input == 1)
    ch = getkeywait(0.5);
    % ch
    if (ch == ch_esc)  % ESC quits the simulation
        break;
    end;
    switch ch
        % d = stop motion
        case ch_d
            if (brain_type == brain_avatared)
                pan_velocity = 0;
                tilt_velocity = 0;
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = 0;
                stim_y_velocity = 0;         
            end;
        % s = move left
        case ch_s
            if (brain_type == brain_avatared)        
                pan_velocity = -pan_speed;
                tilt_velocity = 0; 
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = -stim_speed;
                stim_y_velocity = 0;               
            end;
        % f = move right
        case ch_f
            if (brain_type == brain_avatared)           
                pan_velocity = pan_speed;   
                tilt_velocity = 0; 
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = stim_speed;
                stim_y_velocity = 0;         
            end;
        % e = move up
        case ch_e
            if (brain_type == brain_avatared)   
                pan_velocity = 0;
                tilt_velocity = -tilt_speed;
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = 0;
                stim_y_velocity = -stim_speed;            
            end;
        % c = move down
        case ch_c
            if (brain_type == brain_avatared)   
                pan_velocity = 0;       
                tilt_velocity = tilt_speed;
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = 0;
                stim_y_velocity = stim_speed;                        
            end;
        % w = move up and left
        case ch_w
            if (brain_type == brain_avatared)         
                pan_velocity = -pan_speed;
                tilt_velocity = -tilt_speed; 
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = -stim_speed;
                stim_y_velocity = -stim_speed;                            
            end;
        % r = move up and right
        case ch_r
            if (brain_type == brain_avatared)          
                pan_velocity = pan_speed;
                tilt_velocity = -tilt_speed; 
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = stim_speed;
                stim_y_velocity = -stim_speed;
            end;
        % x = move down and left
        case ch_x
            if (brain_type == brain_avatared)          
                pan_velocity = -pan_speed;
                tilt_velocity = tilt_speed; 
            elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = -stim_speed;
                stim_y_velocity = stim_speed;
            end;
        % v = move down and right
        case ch_v
            if (brain_type == brain_avatared)         
                pan_velocity = pan_speed;
                tilt_velocity = tilt_speed;  
             elseif (stim_behavior == bhvr_avatared)
                stim_x_velocity = stim_speed;
                stim_y_velocity = stim_speed;         
            end;

        % q = toggle reward
        case ch_q
            if (reward_on == 0)
                reward_on = 1;
                disp(['Reward Delivery On. (Iter. ',num2str(iter),')']);
            else
                reward_on = 0;
                disp(['Reward Delivery Off. (Iter. ',num2str(iter),')']);
            end;
        % a = nod command (if not avatared)
        case ch_a
            if (brain_type ~= brain_avatared)
                nod_execute = 1;
            end;
        % z = shake command (if not avatared)
        case ch_z
            if (brain_type ~= brain_avatared)
                shake_execute = 1;
            end;

        % t = avatar the camera
        case ch_t
            if (brain_type == brain_avatared)
                brain_type = brain_rule_based;
                disp(['Camera no longer under user control. (Iter. ', ...
                    num2str(iter),')']);
            else
                brain_type = brain_avatared;
                disp(['Camera now under user control. (Iter. ', ...
                    num2str(iter),')']);            
            end;
        % g = toggle the stimulus on / off
        case ch_g
            if (stim_on == 1)
                stim_on = 0;
            else
                stim_on = 1;
            end;
        % b = cycle through stimulus colors 
        case ch_b
            stim_color_type = stim_color_type + 1;
            if (stim_color_type > color_last)
                stim_color_type = color_blue;
            end;
            if (stim_color_type == color_blue)
                stim_red = 0.0;
                stim_green = 0.0;
                stim_blue = 1.0;  
            elseif (stim_color_type == color_red)
                stim_red = 1.0;
                stim_green = 0.0;
                stim_blue = 0.0;  
            elseif (stim_color_type == color_green)
                stim_red = 0.0;
                stim_green = 1.0;
                stim_blue = 0.0;       
            elseif (stim_color_type == color_yellow) 
                stim_red = 1.0;
                stim_green = 1.0;
                stim_blue = 0.0;          
            elseif (stim_color_type == color_violet)             
                stim_red = 1.0;
                stim_green = 0.0;
                stim_blue = 1.0;        
            end;    

        % 1 = toggle C tone
        case ch_1
            if (tones_on(tone_c) == 0)
                tones_on(tone_c) = 1;
                disp('C Tone On');
            else
                tones_on(tone_c) = 0;
                disp('C Tone Off');           
            end;
        % 2 = toggle Eb tone
        case ch_2
            if (tones_on(tone_eflat) == 0)
                tones_on(tone_eflat) = 1;
                disp('Eb Tone On');
            else
                tones_on(tone_eflat) = 0;
                disp('Eb Tone Off');           
            end;
        % 3 = toggle Gb tone
        case ch_3
            if (tones_on(tone_gflat) == 0)
                tones_on(tone_gflat) = 1;
                disp('Gb Tone On');
            else
                tones_on(tone_gflat) = 0;
                disp('Gb Tone Off');           
            end;
        % 4 = toggle A tone
        case ch_4
            if (tones_on(tone_a) == 0)
                tones_on(tone_a) = 1;
                disp('A Tone On');
            else
                tones_on(tone_a) = 0;
                disp('A Tone Off');           
            end;   

        case ch_j
            curr_event_type = event_c_accept;
            curr_event_timer = 1000;
            tones_on(tone_c) = 1;
        case ch_k
            curr_event_type = event_eflat_reject;
            curr_event_timer = 1000;
            tones_on(tone_eflat) = 1;  
            
        case ch_h
            if (stim_behavior == bhvr_avatared)
                stim_behavior = last_stim_behavior;
                disp(['Stimulus no longer under user control. (Iter. ', ...
                    num2str(iter),')']);
            else
                last_stim_behavior = stim_behavior;
                stim_behavior = bhvr_avatared;
                disp(['Stimulus now under user control. (Iter. ', ...
                    num2str(iter),')']);            
            end;
    end;
end;

% 
% Update the simulation state.
%

% If the event timer has run out, handle the end of event changes and
% switch to a null event.  Otherwise, count the timer down.
if (curr_event_timer <= 0)  
    % Turn off the stimulus.
    stim_on = 0;

    % Turn all tones off.
    tones_on = zeros(12,1);

    % Remember the last event type and previous stimulus behavior, 
    % reset the timer, and set the current event to null.
    last_event_type = curr_event_type;
    prev_stim_behavior = stim_behavior;
    curr_event_timer = 0;
    curr_event_type = event_null;
    
    % Display the end of the event.
    disp(['Event ended: stimuli withdrawn. (Iter. ', num2str(iter),')']);
end;    
    
% If we are in a null event phase, prepare a new stimulus event.
if (curr_event_type == event_null)
    % Figure out what event is being initiated.
    if (simrun_type == simrun_blue_select)
        blue_select_simrun;
    elseif (simrun_type == simrun_red_select)
        red_select_simrun;   
    elseif (simrun_type == simrun_combo_taskset)
        combo_taskset_simrun;
    elseif (simrun_type == simrun_combo_rev_taskset)
        combo_rev_taskset_simrun;        
    elseif (simrun_type == simrun_c_tone_reward)
        c_tone_reward_simrun; 
    elseif (simrun_type == simrun_c_tone_punish)
        c_tone_punish_simrun;         
    end;

    % Process the initiation of the event according to the decided-on type.
    if ((curr_event_type == event_blue_accept) || ...
        (curr_event_type == event_blue_reject))
        stim_on = 1;
        last_stim_start_iter = iter;
        stim_width = size(stim_shape,2);
        stim_height = size(stim_shape,1);
        stim_x = floor(0.5 * scene_width) + 1 - floor(0.5 * stim_width);
        stim_y = floor(0.5 * scene_height) + 1 - floor(0.5 * stim_height);
        if (stim_behavior == bhvr_rnddrop)
            r = rand;
            stim_x = stim_x + 2 * fovea_width - ...
                (floor(4 * fovea_width * r) + 1);
            r = rand;
            stim_y = stim_y + 2 * fovea_height - ...
                (floor(4 * fovea_height * r) + 1);
        end;
        stim_shape_type = shape_square;
        stim_color_type = color_blue;
        stim_red = 0.0;
        stim_green = 0.0;
        stim_blue = 1.0;       
        stim_behavior = stim_all_behavior;
        curr_event_timer = vis_stim_duration;
        stim_was_foveated = 0;
        disp(['Blue square stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');
    end;
    if ((curr_event_type == event_red_accept) || ...
        (curr_event_type == event_red_reject))
        stim_on = 1;   
        last_stim_start_iter = iter;
        stim_width = size(stim_shape,2);
        stim_height = size(stim_shape,1);
        stim_x = floor(0.5 * scene_width) + 1 - floor(0.5 * stim_width);
        stim_y = floor(0.5 * scene_height) + 1 - floor(0.5 * stim_height);
        if (stim_behavior == bhvr_rnddrop)
            r = rand;
            stim_x = stim_x + 2 * fovea_width - ...
                (floor(4 * fovea_width * r) + 1);
            r = rand;
            stim_y = stim_y + 2 * fovea_height - ...
                (floor(4 * fovea_height * r) + 1);
        end;
        stim_shape_type = shape_square;
        stim_color_type = color_red;
        stim_red = 1.0;
        stim_green = 0.0;
        stim_blue = 0.0;     
        stim_behavior = stim_all_behavior;        
        curr_event_timer = vis_stim_duration;
        stim_was_foveated = 0;
        disp(['Red square stimulus spawned. (Iter. ', num2str(iter),')']); 
        disp('------------------------------------------------------');        
    end;   
    if ((curr_event_type == event_green_accept) || ...
        (curr_event_type == event_green_reject))
        stim_on = 1;   
        last_stim_start_iter = iter;
        stim_width = size(stim_shape,2);
        stim_height = size(stim_shape,1);
        stim_x = floor(0.5 * scene_width) + 1 - floor(0.5 * stim_width);
        stim_y = floor(0.5 * scene_height) + 1 - floor(0.5 * stim_height);
        if (stim_behavior == bhvr_rnddrop)
            r = rand;
            stim_x = stim_x + 2 * fovea_width - ...
                (floor(4 * fovea_width * r) + 1);
            r = rand;
            stim_y = stim_y + 2 * fovea_height - ...
                (floor(4 * fovea_height * r) + 1);
        end;
        stim_shape_type = shape_square;
        stim_color_type = color_green;
        stim_red = 0.0;
        stim_green = 1.0;
        stim_blue = 0.0;     
        stim_behavior = stim_all_behavior;        
        curr_event_timer = vis_stim_duration;
        stim_was_foveated = 0;
        disp(['Green square stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end;   
    if ((curr_event_type == event_yellow_accept) || ...
        (curr_event_type == event_yellow_reject))
        stim_on = 1;   
        last_stim_start_iter = iter;
        stim_width = size(stim_shape,2);
        stim_height = size(stim_shape,1);
        stim_x = floor(0.5 * scene_width) + 1 - floor(0.5 * stim_width);
        stim_y = floor(0.5 * scene_height) + 1 - floor(0.5 * stim_height);
        if (stim_behavior == bhvr_rnddrop)
            r = rand;
            stim_x = stim_x + 2 * fovea_width - ...
                (floor(4 * fovea_width * r) + 1);
            r = rand;
            stim_y = stim_y + 2 * fovea_height - ...
                (floor(4 * fovea_height * r) + 1);
        end;
        stim_shape_type = shape_square;
        stim_color_type = color_yellow;
        stim_red = 1.0;
        stim_green = 1.0;
        stim_blue = 0.0;     
        stim_behavior = stim_all_behavior;        
        curr_event_timer = vis_stim_duration;
        stim_was_foveated = 0;
        disp(['Yellow square stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end;   
    if ((curr_event_type == event_violet_accept) || ...
        (curr_event_type == event_violet_reject))
        stim_on = 1;   
        last_stim_start_iter = iter;
        stim_width = size(stim_shape,2);
        stim_height = size(stim_shape,1);
        stim_x = floor(0.5 * scene_width) + 1 - floor(0.5 * stim_width);
        stim_y = floor(0.5 * scene_height) + 1 - floor(0.5 * stim_height);
        if (stim_behavior == bhvr_rnddrop)
            r = rand;
            stim_x = stim_x + 2 * fovea_width - ...
                (floor(4 * fovea_width * r) + 1);
            r = rand;
            stim_y = stim_y + 2 * fovea_height - ...
                (floor(4 * fovea_height * r) + 1);
        end;
        stim_shape_type = shape_square;
        stim_color_type = color_violet;
        stim_red = 1.0;
        stim_green = 0.0;
        stim_blue = 1.0;     
        stim_behavior = stim_all_behavior;        
        curr_event_timer = vis_stim_duration;
        stim_was_foveated = 0;
        disp(['Violet square stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end;       
    if (curr_event_type == event_delay)
        curr_event_timer = delay_duration;
        reinf_delay_on = 1;
        reinf_delay_timer = reinf_delay_dur;
        disp(['Delay period begun. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');  
        delay_count = delay_count + 1;
    end;
    if ((curr_event_type == event_c_accept) || ...
        (curr_event_type == event_c_reject) || ...
        (curr_event_type == event_c_noresp) || ...  
        (curr_event_type == event_c_delrew) || ...          
        (curr_event_type == event_c_delpun))            
        stim_on = 0;
        last_stim_start_iter = iter;
        tones_on(tone_c) = 1;     
        curr_event_timer = tone_stim_duration;
        disp(['C tone stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end;
    if ((curr_event_type == event_eflat_accept) || ...
        (curr_event_type == event_eflat_reject) || ...
        (curr_event_type == event_eflat_noresp) || ...
        (curr_event_type == event_eflat_delrew) || ...         
        (curr_event_type == event_eflat_delpun))            
        stim_on = 0;
        last_stim_start_iter = iter;
        tones_on(tone_eflat) = 1;     
        curr_event_timer = tone_stim_duration;
        disp(['Eb tone stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end; 
    if ((curr_event_type == event_gflat_accept) || ...
        (curr_event_type == event_gflat_reject) || ...
        (curr_event_type == event_gflat_noresp) || ... 
        (curr_event_type == event_gflat_delrew) || ...         
        (curr_event_type == event_gflat_delpun))            
        stim_on = 0;
        last_stim_start_iter = iter;
        tones_on(tone_gflat) = 1;     
        curr_event_timer = tone_stim_duration;
        disp(['Gb tone stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end; 
    if ((curr_event_type == event_a_accept) || ...
        (curr_event_type == event_a_reject) || ...
        (curr_event_type == event_a_noresp) || ... 
        (curr_event_type == event_a_delrew) || ...         
        (curr_event_type == event_a_delpun))            
        stim_on = 0;
        last_stim_start_iter = iter;
        tones_on(tone_a) = 1;     
        curr_event_timer = tone_stim_duration;
        disp(['A tone stimulus spawned. (Iter. ', num2str(iter),')']);
        disp('------------------------------------------------------');        
    end; 
    
    if ((curr_event_type == event_blue_accept) || ...
        (curr_event_type == event_red_accept) || ...
        (curr_event_type == event_green_accept) || ...
        (curr_event_type == event_yellow_accept) || ...
        (curr_event_type == event_violet_accept) || ...
        (curr_event_type == event_c_accept) || ...
        (curr_event_type == event_eflat_accept))
        reward_trigger = rtrigger_nod;
        reward_delivered = 0;
        punish_trigger = punish_strictness;
        punish_delivered = 0;
        target_count = target_count + 1;
    end;
    
    if ((curr_event_type == event_blue_reject) || ...
        (curr_event_type == event_red_reject) || ...
        (curr_event_type == event_green_reject) || ...
        (curr_event_type == event_yellow_reject) || ...
        (curr_event_type == event_violet_reject) || ...
        (curr_event_type == event_c_reject) || ...
        (curr_event_type == event_eflat_reject))
        reward_trigger = rtrigger_shake;
        reward_delivered = 0;
        punish_trigger = punish_strictness;
        punish_delivered = 0;
        distractor_count = distractor_count + 1;
    end;   

    if ((curr_event_type == event_c_delrew) || ...
        (curr_event_type == event_eflat_delrew) || ...
        (curr_event_type == event_gflat_delrew) || ...
        (curr_event_type == event_a_delrew))   
        reward_trigger = rtrigger_delay;
        reward_delivered = 0;
        stim_rew_delay_timer = stim_rew_delay_dur;
    end;

    if ((curr_event_type == event_c_delpun) || ...
        (curr_event_type == event_eflat_delpun) || ...
        (curr_event_type == event_gflat_delpun) || ...
        (curr_event_type == event_a_delpun))   
        punish_trigger = ptrigger_delay;
        punish_delivered = 0;
        stim_pun_delay_timer = stim_pun_delay_dur;
    end;
    
    % Increment the events counter, if we don't just have another null
    % event.
    if (curr_event_type ~= event_null)
        events_count = events_count + 1;
    end;
end;

% Default to not delivering a reward or a punisher.
do_reward = 0;
do_punish = 0;

% Terminate any expired rewards (and with them working memory traces for 
% colors and tones).
if ((reward_on == 1) && (reward_timer <= 0))
    disp(['Reward delivery terminated. (Iter. ',num2str(iter),')']);
    reward_on = 0;
    reward_timer = 0;
%     mem_color_map = zeros(4,1);
%     mem_tones_map = zeros(12,1);
end;

% If the post-stimulus reinforcement delivery window timer expires, turn 
% off the potential delivery of a reward or punisher.
if ((reinf_delay_on == 1) && (reinf_delay_timer <= 0))
    reinf_delay_on = 0;
    reinf_delay_timer = 0;
    
    % Build a list of the kinds of errors/responses that could invalidate a
    % miss. 
    miss_inval_resps = [ target_nod target_shake target_x_nod ...
        target_nod_shake distractor_nod distractor_shake ...
        distractor_x_shake distractor_nod_shake blind_nod blind_shake ];
    
    % If none of these follow the beginning of the last stimulus, and we 
    % are not nodding or shaking, we can flag a miss.
    if ((isempty(find(miss_inval_resps > last_stim_start_iter))) && ...
        (nod_execute == 0) && (shake_execute == 0))
        if (reward_trigger == rtrigger_nod)
            disp(['TARGET MISS (BAD). (Iter. ',num2str(iter),')']);
            target_miss = [ target_miss iter ];
        elseif (reward_trigger == rtrigger_shake)
            disp(['DISTRACTOR MISS (BAD). (Iter. ',num2str(iter),')']);
            distractor_miss = [ distractor_miss iter ];
        end;
        if (punish_misses == 1)
            do_punish = 1;
        end;
    end;  
    reward_trigger = rtrigger_none;
    mem_color_map = zeros(4,1);
end;

% Terminate any expired punishers (and with them working memory traces for 
% colors and tones).
if ((punish_on == 1) && (punish_timer <= 0))
    disp(['Punisher delivery terminated. (Iter. ',num2str(iter),')']);
    punish_on = 0;
    punish_timer = 0;
%     mem_color_map = zeros(4,1);
%     mem_tones_map = zeros(12,1);
end;

% If we are due to deliver a delayed stimulus-cued reward, do so, and reset
% the reward delivery.
if ((reward_trigger == rtrigger_delay) && (stim_rew_delay_timer <= 0))
    stim_rew_delay_timer = 0;  
    reward_trigger = rtrigger_none;   
    do_reward = 1;
end;

% If we are due to deliver a delayed stimulus-cued punisher, do so, and 
% reset the punisher delivery.
if ((punish_trigger == ptrigger_delay) && (stim_pun_delay_timer <= 0))
    stim_pun_delay_timer = 0;  
    punish_trigger = ptrigger_none;   
    do_punish = 1;
end;

% Look for a Nod and if one is detected process consequences, including 
% reward and/or tabulation of hits or misses.
if (nod_iter_count == nod_seen_thresh)
    disp(['Nod detected in environment. (Iter. ',num2str(iter),')']);  
    
    % Do not reward cases where the Nod overlaps with a Shake.
    if ((shake_execute ~= 0) || (last_shake_end_iter > last_nod_start_iter))
        % Only flag/punish case where Shake is still executing...
        if (shake_execute ~= 0)
            if (reward_trigger == rtrigger_nod)
                if (punish_nodshakes == 1)
                    do_punish = 1;
                end;
                disp(['TARGET NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);           
                target_nod_shake = [ target_nod_shake iter ]; 
            elseif (reward_trigger == rtrigger_shake)
                if (punish_nodshakes == 1)
                    do_punish = 1;
                end;            
                disp(['DISTRACTOR NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);
                distractor_nod_shake = [ distractor_nod_shake iter ];
            else
                if (punish_delays == 1)
                    do_punish = 1;
                end;             
                disp(['DELAY NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);
                delay_nod_shake = [ delay_nod_shake iter ];            
            end;
        end;
    % Do not reward cases where the Nod began before the stimulus was
    % delivered.
    elseif (last_nod_start_iter < last_stim_start_iter)
        if (punish_delays == 1)
            do_punish = 1;
        end;         
        disp(['DELAY NOD (BAD). (Iter. ',num2str(iter),')']);
        delay_nod = [ delay_nod iter ];  
    elseif (reward_trigger == rtrigger_nod)
        % Do not reward responses to visual stimuli that are made without
        % having looked at them (if the requirement is present).  NOTE:
        % This code will need to be changed if response to auditory stimuli
        % is to be rewarded.   
        if ((require_foveate == 1) && (stim_was_foveated == 0))
            if (punish_blinds == 1)
                do_punish = 1;
            end;              
            disp(['BLIND NOD (BAD). (Iter. ',num2str(iter),')']);           
            blind_nod = [ blind_nod iter ];            
        elseif (reward_delivered == 0)
            do_reward = 1;        
            disp(['TARGET NOD (GOOD). (Iter. ',num2str(iter),')']);
            target_nod = [ target_nod iter ];
        else
            if (punish_extras == 1)
                do_punish = 1;
            end;               
            disp(['TARGET EXTRA NOD (BAD). (Iter. ',num2str(iter),')']);           
            target_x_nod = [ target_x_nod iter ];
        end;         
    elseif (reward_trigger == rtrigger_shake)
        if (punish_opposites == 1)
            do_punish = 1;
        end;          
        disp(['DISTRACTOR NOD (BAD). (Iter. ',num2str(iter),')']);
        distractor_nod = [ distractor_nod iter ];
    else
        if (punish_delays == 1)
            do_punish = 1;
        end;          
        disp(['DELAY NOD (BAD). (Iter. ',num2str(iter),')']);
        delay_nod = [ delay_nod iter ];            
    end;   
end;

% Look for a Shake and if one is detected process consequences, including 
% reward and/or tabulation of hits or misses.
if (shake_iter_count == shake_seen_thresh)
    disp(['Shake detected in environment. (Iter. ',num2str(iter),')']);
    
    % Do not reward cases where the Shake overlaps with a Nod.
    if ((nod_execute ~= 0) || (last_nod_end_iter > last_shake_start_iter))
        % Only flag/punish case where Nod is still executing...
        if (nod_execute ~= 0)        
            if (reward_trigger == rtrigger_nod)
                if (punish_nodshakes == 1)
                    do_punish = 1;
                end;           
                disp(['TARGET NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);           
                target_nod_shake = [ target_nod_shake iter ]; 
            elseif (reward_trigger == rtrigger_shake)
                if (punish_nodshakes == 1)
                    do_punish = 1;
                end;           
                disp(['DISTRACTOR NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);
                distractor_nod_shake = [ distractor_nod_shake iter ];
            else
                if (punish_delays == 1)
                    do_punish = 1;
                end;            
                disp(['DELAY NOD-SHAKE (BAD). (Iter. ',num2str(iter),')']);
                delay_nod_shake = [ delay_nod_shake iter ];            
            end;
        end;
    % Do not reward cases where the Shake began before the stimulus was
    % delivered.
    elseif (last_shake_start_iter < last_stim_start_iter)
        if (punish_delays == 1)
            do_punish = 1;
        end;            
        disp(['DELAY SHAKE (BAD). (Iter. ',num2str(iter),')']);
        delay_shake = [ delay_shake iter ];        
    elseif (reward_trigger == rtrigger_shake)
        % Do not reward responses to visual stimuli that are made without
        % having looked at them (if the requirement is present).  NOTE:
        % This code will need to be changed if response to auditory stimuli
        % is to be rewarded.
        if ((require_foveate == 1) && (stim_was_foveated == 0))
            if (punish_blinds == 1)
                do_punish = 1;
            end;             
            disp(['BLIND SHAKE (BAD). (Iter. ',num2str(iter),')']);           
            blind_shake = [ blind_shake iter ];          
        elseif (reward_delivered == 0)
            do_reward = 1;
            disp(['DISTRACTOR SHAKE (GOOD). (Iter. ',num2str(iter),')']);
            distractor_shake = [ distractor_shake iter ];
        else
            if (punish_extras == 1)
                do_punish = 1;
            end;             
            disp(['DISTRACTOR EXTRA SHAKE (BAD). (Iter. ',num2str(iter),')']);
            distractor_x_shake = [ distractor_x_shake iter ];
        end;           
    elseif (reward_trigger == rtrigger_nod)
        if (punish_opposites == 1)
            do_punish = 1;
        end;            
        disp(['TARGET SHAKE (BAD). (Iter. ',num2str(iter),')']);
        target_shake = [ target_shake iter ];
    else
        if (punish_delays == 1)
            do_punish = 1;
        end;            
        disp(['DELAY SHAKE (BAD). (Iter. ',num2str(iter),')']);
        delay_shake = [ delay_shake iter ];            
    end;    
end;

% If we need to deliver a reward.
if (do_reward == 1)
    r = rand;         
    if (r * 100 < reward_delivery_pct)
        disp(['Reward delivery begun. (Iter. ',num2str(iter),')']);          
        reward_on = 1;
        reward_count = reward_count + 1;
        reward_timer = reward_duration; 
        reward_delivered = 1;
    else
        disp(['Reward omitted. (Iter. ',num2str(iter),')']);
    end;
    do_reward = 0;
end;
    
% If we need to deliver a punisher.
if (do_punish == 1)
    r = rand;     
    if (r * 100 < punish_delivery_pct)
        disp(['Punisher delivery begun. (Iter. ',num2str(iter),')']);          
        punish_on = 1;
        punish_count = punish_count + 1;
        punish_timer = punish_duration; 
        punish_delivered = 1;
    else
        disp(['Punisher omitted. (Iter. ',num2str(iter),')']);
    end;
    do_punish = 0;
end;

% Count down the reward timer if a reward is active.
if (reward_on == 1)
    reward_timer = reward_timer - iter_duration;
end;

% Count down the post-stimulus reinforcement delivery window timer.
if (reinf_delay_on == 1)
    reinf_delay_timer = reinf_delay_timer - iter_duration;
end;

% Count down the stimulus delayed reward timer.
if (reward_trigger == rtrigger_delay)
    stim_rew_delay_timer = stim_rew_delay_timer - iter_duration;
end;

% Count down the punisher timer if a punisher is active.
if (punish_on == 1)
    punish_timer = punish_timer - iter_duration;
end;

% Count down the stimulus delayed punisher timer.
if (punish_trigger == ptrigger_delay)
    stim_pun_delay_timer = stim_pun_delay_timer - iter_duration;
end;

% If we are in a non-null event and the behavior of the stimulus isn't being 
% avatared, update the event timer.
if ((curr_event_type ~= event_null) && (stim_behavior ~= bhvr_avatared))
    curr_event_timer = curr_event_timer - iter_duration;
end;

% If the stimulus is present, update its location and make sure it is
% within the bounds of the scene.
if (stim_on == 1)
    % Update the stimulus' location according to its current speed.
    stim_x = stim_x + stim_x_velocity * iter_duration / 1000;
    stim_y = stim_y + stim_y_velocity * iter_duration / 1000;

    % Keep the stimulus in bounds if it went out of bounds.
    if (stim_x < 1)
        stim_x = 1;
    end;
    if (stim_x > (scene_width - size(stim_shape,2) + 1))
        stim_x = scene_width - size(stim_shape,2) + 1;
    end;
    if (stim_y < 1)
        stim_y = 1;
    end;
    if (stim_y > (scene_height - size(stim_shape,1) + 1))
        stim_y = scene_height - size(stim_shape,1) + 1;
    end;
end;

% Update the whole scene according to the stimulus' new position.
% whole_scene is what the camera is able to see.
whole_scene = zeros(scene_height,scene_width,3);
if (stim_on == 1)
    for xx = 1:size(stim_shape,2)
        for yy = 1:size(stim_shape,1)
            whole_scene(floor(stim_y) + yy - 1, floor(stim_x) + xx - 1, 1) = ...
                stim_red * stim_shape(yy,xx);
            whole_scene(floor(stim_y) + yy - 1, floor(stim_x) + xx - 1, 2) = ...
                stim_green * stim_shape(yy,xx);
            whole_scene(floor(stim_y) + yy - 1, floor(stim_x) + xx - 1, 3) = ...
                stim_blue * stim_shape(yy,xx);
        end;
    end;
end;

%
% Update the body (camera) state (except for the speed which is set by the 
% brain unless the camera is avatared).
%

% Update the camera's location according the current speed.
camera_x = camera_x + pan_velocity * iter_duration / 1000;
camera_y = camera_y + tilt_velocity * iter_duration / 1000;

% Keep the camera in bounds if it went out of bounds.
if (camera_x < 1)
    camera_x = 1;
end;
if (camera_x > (scene_width - camera_width + 1))
    camera_x = scene_width - camera_width + 1;
end;
if (camera_y < 1)
    camera_y = 1;
end;
if (camera_y > (scene_height - camera_height + 1))
    camera_y = scene_height - camera_height + 1;
end;

%
% Update the brain of the camera.
%

% Set camera and fovea boundaries according to new position.
camera_left_b = floor(camera_x);
camera_right_b = floor(camera_x) + camera_width - 1;
camera_top_b = floor(camera_y);
camera_bottom_b = floor(camera_y) + camera_height - 1;
fovea_left_b = floor(camera_x) + (camera_width - fovea_width) / 2.0;
fovea_right_b = fovea_left_b + fovea_width - 1;
fovea_top_b = floor(camera_y) + (camera_height - fovea_height) / 2.0;
fovea_bottom_b = fovea_top_b + fovea_height - 1;

% Update the retina rods to the maximum of the colors in the camera window 
% on the scene.
for xx = 1:camera_width
    for yy = 1:camera_height
        rod_val(1) = whole_scene(camera_top_b + yy - 1, ...
            camera_left_b + xx - 1, 1);
        rod_val(2) = whole_scene(camera_top_b + yy - 1, ...
            camera_left_b + xx - 1, 2);
        rod_val(3) = whole_scene(camera_top_b + yy - 1, ...
            camera_left_b + xx - 1, 3);        
        retina_rods(yy,xx) = max(rod_val);
    end;
end;

% Update the retina cones to be the values seen in the foveal portion of 
% the camera.
for xx = 1:fovea_width
    for yy = 1:fovea_height
        for kk = 1:3
            retina_cones(yy,xx,kk) = whole_scene(fovea_top_b + yy - 1, ...
                fovea_left_b + xx - 1, kk);
        end;
    end;
end;

% Remember the previous color map, so we can do comparisons.
past_color_map = color_map;

% Set the color map units according to the color opponencies red vs. green 
% and blue vs. yellow.
red_mean = mean(mean(retina_cones(:,:,1)));
green_mean = mean(mean(retina_cones(:,:,2)));
blue_mean = mean(mean(retina_cones(:,:,3)));
color_map(1) = red_mean - green_mean;  % Red+/Green- (red detector)
color_map(2) = green_mean - red_mean;  % Green+/Red- (green detector)
color_map(3) = blue_mean - 0.5 * (red_mean + green_mean);
                                       % Blue+/Yellow- (blue detector)
color_map(4) = 0.5 * (red_mean + green_mean) - blue_mean;
                                       % Yellow+/Blue- (yellow detector)
for k = 1:4
    if (color_map(k) < 0)
        color_map(k) = 0;
    end;
end;
if (sum(color_map) > 0.3)
    stim_was_foveated = 1;
end;

% If color working memory is expired, turn it off.
if (color_mem_timer <= 0)
    mem_color_map = zeros(4,1);
end;

% Remember the previous tones, so we can do comparisons.
past_tones_map = tones_map;

% Read the tone stimuli into the brain tones map.
tones_map = tones_on;

% If tones working memory is expired, turn it off.
if (tones_mem_timer <= 0)
    mem_tones_map = zeros(12,1);
end;

% Set the memory color map if the color map reflects new color information.
if ((sum(mem_color_map == (color_map > 0.5)) < 4) && ...
        (sum(color_map > 0.5) > 0))
    mem_color_map = (color_map > 0.5);
    color_mem_timer = color_mem_duration;
end;

% Set the tones working memory if the current tones reflect new information.
if ((sum(mem_tones_map == (tones_map > 0.5)) < 12) && ...
        (sum(tones_map > 0.5) > 0))
    mem_tones_map = (tones_map > 0.5);
    tones_mem_timer = tones_mem_duration;
end;

% If color working memory is engaged, decrement its timer.
if (color_mem_timer > 0)
    color_mem_timer = color_mem_timer - iter_duration;
end;

% If color working memory is engaged, decrement its timer.
if (tones_mem_timer > 0)
    tones_mem_timer = tones_mem_timer - iter_duration;
end;

% Terminate any expired novelty signals. 
if ((novelty_on == 1) && (novelty_timer <= 0))
    disp(['Novelty signal terminated. (Iter. ',num2str(iter),')']);
    novelty_on = 0;
    novelty_timer = 0;
end;

% Initiate a novelty signal on an onset of a color.
for k = 1:4
    if (novcount_color_map(k) > 0)
        if (((k < 4) && (past_color_map(k) < 0.3) && (color_map(k) > 0.3)) || ...
            ((k == 4) && (past_color_map(k) < 0.6) && (color_map(k) > 0.6)))
            novcount_color_map(k) = novcount_color_map(k) - 1;
            novelty_on = 1;
            novelty_timer = novelty_duration;  
        end;
    end;
end;

% Initiate a novelty signal on an onset of a tone.
for k = 1:12
    if (novcount_tones(k) > 0)
        if ((past_tones_map(k) < 0.3) && (tones_map(k) > 0.3))
            novcount_tones(k) = novcount_tones(k) - 1;
            novelty_on = 1;
            novelty_timer = novelty_duration;  
        end;
    end;
end;
            
% Count down the novelty signal timer if a novelty signal is active.
if (novelty_on == 1)
    novelty_timer = novelty_timer - iter_duration;
end;


% Process the activity in the neural parts of the model (null, rule-based, 
% or actual neural units.  If the brain (camera) is avatared, nothing
% happens here.

% Call the rule-based script if we're running a rule-based model.
if (brain_type == brain_rule_based)
    rule_based_brain;
elseif (brain_type == brain_cog_model)
    cog_model_brain;
end;

% Set the camera speed according to the brain's output if the camera is not 
% being avatared.
if (brain_type ~= brain_avatared)
    % Initialize the velocity components to zero.
    pan_velocity = 0;
    tilt_velocity = 0;
    
    % By default, the camera organism is always tracking.  If a Nod or
    % Shake is happening, though, this is inhibited.
    track_execute = 1;
    if ((nod_execute == 1) || (shake_execute == 1))
        track_execute = 0;
    end;
   
    % Perform the tracking behavior if it is active.
    if (track_execute == 1)
        motor_input_thresh = 0.06;
        stim_in_center_thresh = 0.5;
        left_motor_input = mean(mean(retina_rods(:,1:24)));
        right_motor_input = mean(mean(retina_rods(:,41:64)));
        up_motor_input = mean(mean(retina_rods(1:24,:)));
        down_motor_input = mean(mean(retina_rods(41:64,:)));
        stim_center_input = mean(mean(retina_rods(25:40,25:40)));
        if (left_motor_input > motor_input_thresh)
            pan_velocity = pan_velocity - pan_speed;
        end;
        if (right_motor_input > motor_input_thresh)
            pan_velocity = pan_velocity + pan_speed;
        end;
        if (up_motor_input > motor_input_thresh)
            tilt_velocity = tilt_velocity - tilt_speed;
        end; 
        if (down_motor_input > motor_input_thresh)
            tilt_velocity = tilt_velocity + tilt_speed;
        end;  
       
        % Center the camera if no stimulus is seen.
        if ((pan_velocity == 0) && (tilt_velocity == 0) && ...
                (stim_center_input < stim_in_center_thresh))
            if (camera_x < (1 + (scene_width - camera_width) / 2.0) - 5)
                pan_velocity = pan_speed;
            elseif (camera_x > (1 + (scene_width - camera_width) / 2.0) + 5)
                pan_velocity = -pan_speed;
            end;
            if (camera_y < (1 + (scene_height - camera_height) / 2.0) - 5)
                tilt_velocity = tilt_speed;
            elseif (camera_y > (1 + (scene_height - camera_height) / 2.0) + 5)
                tilt_velocity = -tilt_speed;
            end;
        end;
    end;
    
    % Perform the Nod behavior if it is active.
    if (nod_execute == 1)
        if (nod_centered == 0)
            pan_velocity_mod = 0;
            tilt_velocity_mod = 0;
            if (camera_x < (1 + (scene_width - camera_width) / 2.0) - 5)
                pan_velocity_mod = pan_speed;
            elseif (camera_x > (1 + (scene_width - camera_width) / 2.0) + 5)
                pan_velocity_mod = -pan_speed;
            end;
            if (camera_y < (1 + (scene_height - camera_height) / 2.0) - 5)
                tilt_velocity_mod = tilt_speed;
            elseif (camera_y > (1 + (scene_height - camera_height) / 2.0) + 5)
                tilt_velocity_mod = -tilt_speed;
            end;
            if ((pan_velocity_mod == 0) && (tilt_velocity_mod == 0))
                nod_centered = 1;
                nod_iter_count = 0;
                nod_phase = 1;
            else
                pan_velocity = pan_velocity + pan_velocity_mod;
                tilt_velocity = tilt_velocity + tilt_velocity_mod;
            end;
        end;
        if (nod_centered == 1)
            nod_iter_count = nod_iter_count + 1;
            if (nod_iter_count == 1)
                nod_phase = 1;
                last_nod_start_iter = iter;  % remember when the nod started
            elseif (nod_iter_count == 3)
                nod_phase = 2;
            elseif (nod_iter_count == 7)
                nod_phase = 3;
            elseif (nod_iter_count == 11)
                nod_phase = 4;
            elseif (nod_iter_count == 15)
                nod_phase = 5;
            elseif (nod_iter_count == 17)
                nod_phase = 0;
                nod_execute = 0;
                nod_centered = 0;
                last_nod_end_iter = iter;  % remember when the nod ended
            end;
            if (nod_phase == 1)
                tilt_velocity = tilt_velocity - tilt_speed;
            elseif (nod_phase == 2)
                tilt_velocity = tilt_velocity + tilt_speed;
            elseif (nod_phase == 3)
                tilt_velocity = tilt_velocity - tilt_speed;
            elseif (nod_phase == 4)
                tilt_velocity = tilt_velocity + tilt_speed;
            elseif (nod_phase == 5)
                tilt_velocity = tilt_velocity - tilt_speed;
            end;                
        end;
    end;
    
    % Perform the Shake behavior if it is active.
    if (shake_execute == 1)
        if (shake_centered == 0)
            pan_velocity_mod = 0;
            tilt_velocity_mod = 0;
            if (camera_x < (1 + (scene_width - camera_width) / 2.0) - 5)
                pan_velocity_mod = pan_speed;
            elseif (camera_x > (1 + (scene_width - camera_width) / 2.0) + 5)
                pan_velocity_mod = -pan_speed;
            end;
            if (camera_y < (1 + (scene_height - camera_height) / 2.0) - 5)
                tilt_velocity_mod = tilt_speed;
            elseif (camera_y > (1 + (scene_height - camera_height) / 2.0) + 5)
                tilt_velocity_mod = -tilt_speed;
            end;
            if ((pan_velocity_mod == 0) && (tilt_velocity_mod == 0))
                shake_centered = 1;
                shake_iter_count = 0;
                shake_phase = 1;
            elseif (nod_execute == 0)  % only if not Nod/Shake
                pan_velocity = pan_velocity + pan_velocity_mod;
                tilt_velocity = tilt_velocity + tilt_velocity_mod;
            end;
        end;
        if (shake_centered == 1)      
            shake_iter_count = shake_iter_count + 1;
            if (shake_iter_count == 1)
                shake_phase = 1;
                last_shake_start_iter = iter;  % remember when the shake started
            elseif (shake_iter_count == 3)
                shake_phase = 2;
            elseif (shake_iter_count == 7)
                shake_phase = 3;
            elseif (shake_iter_count == 11)
                shake_phase = 4;
            elseif (shake_iter_count == 15)
                shake_phase = 5;
            elseif (shake_iter_count == 17)
                shake_phase = 0;
                shake_execute = 0;
                shake_centered = 0;
                last_shake_end_iter = iter;  % remember when the shake ended
            end;
            if (shake_phase == 1)
                pan_velocity = pan_velocity - pan_speed;
            elseif (shake_phase == 2)
                pan_velocity = pan_velocity + pan_speed;
            elseif (shake_phase == 3)
                pan_velocity = pan_velocity - pan_speed;
            elseif (shake_phase == 4)
                pan_velocity = pan_velocity + pan_speed;
            elseif (shake_phase == 5)
                pan_velocity = pan_velocity - pan_speed;
            end;                
        end;        
    end;    
end;

%
% Render the scene for display.
%

% If we are to display the scene...
if (display_scene == 1)
    % Add a pause if we have no keyboard input (so we can see output).
    if (get_key_input == 0)
        pause(0.001);
    end;
    
    % Set the display to what's there behind the camera.
    disp_scene = whole_scene;

    % Draw camera boundaries.
    for xx = camera_left_b : camera_right_b
        disp_scene(camera_top_b, xx, 1) = 1.0;
        disp_scene(camera_top_b, xx, 2) = 1.0;
        disp_scene(camera_top_b, xx, 3) = 1.0;    
        disp_scene(camera_bottom_b, xx, 1) = 1.0;
        disp_scene(camera_bottom_b, xx, 2) = 1.0;
        disp_scene(camera_bottom_b, xx, 3) = 1.0;
    end;
    for yy = camera_top_b : camera_bottom_b
        disp_scene(yy, camera_left_b, 1) = 1.0;
        disp_scene(yy, camera_left_b, 2) = 1.0;
        disp_scene(yy, camera_left_b, 3) = 1.0;
        disp_scene(yy, camera_right_b, 1) = 1.0;
        disp_scene(yy, camera_right_b, 2) = 1.0;
        disp_scene(yy, camera_right_b, 3) = 1.0;
    end;

    % Draw fovea boundaries.
    for xx = fovea_left_b : fovea_right_b
        disp_scene(fovea_top_b, xx, 1) = 1.0;
        disp_scene(fovea_top_b, xx, 2) = 1.0;
        disp_scene(fovea_top_b, xx, 3) = 1.0;
        disp_scene(fovea_bottom_b, xx, 1) = 1.0;
        disp_scene(fovea_bottom_b, xx, 2) = 1.0;
        disp_scene(fovea_bottom_b, xx, 3) = 1.0;
    end;
    for yy = fovea_top_b : fovea_bottom_b
        disp_scene(yy, fovea_left_b, 1) = 1.0;
        disp_scene(yy, fovea_left_b, 2) = 1.0;
        disp_scene(yy, fovea_left_b, 3) = 1.0;
        disp_scene(yy, fovea_right_b, 1) = 1.0;
        disp_scene(yy, fovea_right_b, 2) = 1.0;
        disp_scene(yy, fovea_right_b, 3) = 1.0;
    end;
    
    % Draw the environment and camaera.
    figtitle = strcat('Iteration #',num2str(iter));
    set(fh, 'Name', figtitle);
    plot_id = 1;
    if (subplot_inds(plot_id,3) > 0)
        axes(env_axes);
        set(env_im,'CData',disp_scene);
    end;

    % Draw the perifoveal view.
    plot_id = 2;
    if (subplot_inds(plot_id,3) > 0)
        axes(perifov_axes);
        set(perifov_im,'CData',retina_rods);
    end;

    % Draw the motor exec units.
    plot_id = 3;
    if (subplot_inds(plot_id,3) > 0)
        motor_units = zeros(3,1);
        if (brain_type == brain_cog_model)
            area_ind = TrackExec_ind; 
            fetch_area_act; 
            motor_units(1) = activs;
            area_ind = NodExec_ind; 
            fetch_area_act;     
            motor_units(2) = activs;
            area_ind = ShakeExec_ind; 
            fetch_area_act;       
            motor_units(3) = activs;
        else
            motor_units(1) = track_execute;
            motor_units(2) = nod_execute;
            motor_units(3) = shake_execute;
        end;
        axes(motor_exec_axes);
        set(motor_exec_im,'CData',motor_units);
    end;

    % Draw the color input units.
    plot_id = 4;
    if (subplot_inds(plot_id,3) > 0)
        axes(color_input_axes);
        if (brain_type == brain_cog_model)
            area_ind = ColorIn_ind; 
            fetch_area_act;
            set(color_input_im,'CData',[activs mem_color_map]);
        else
            set(color_input_im,'CData',[color_map mem_color_map]);      
        end;
    end;

    % Draw the tone input units.
    plot_id = 5;
    if (subplot_inds(plot_id,3) > 0)
        axes(tones_input_axes);
        if (brain_type == brain_cog_model)
            area_ind = TonesIn_ind; 
            fetch_area_act; 
            set(tones_input_im,'CData',[activs mem_tones_map]);
        else 
            set(tones_input_im,'CData',[tones_on mem_tones_map]); 
        end;
    end;

    % Draw the reward unit.
    plot_id = 6;
    if (subplot_inds(plot_id,3) > 0)
        axes(reward_input_axes);
        if (brain_type == brain_cog_model)
            area_ind = RewardIn_ind; 
            fetch_area_act;  
            set(reward_input_im,'CData',activs);
        else     
            set(reward_input_im,'CData',reward_on);
        end;
        if ((reward_trigger == rtrigger_nod) && (reward_delivered == 0))
            xlabel('Reward Unit (On Nod)');
        elseif ((reward_trigger == rtrigger_shake) && (reward_delivered == 0))
            xlabel('Reward Unit (On Shake)');
        else
            xlabel('Reward Unit');
        end;
    end;

    if (brain_type == brain_cog_model)
        % Draw the plan units.
        plot_id = 7;
        if (subplot_inds(plot_id,3) > 0)
            axes(plan_units_axes);
            area_ind = Plan_ind; 
            fetch_area_act; 
            set(plan_units_im,'CData',activs);  
        end;

        % Draw the output (motor init) units.
        plot_id = 8;
        if (subplot_inds(plot_id,3) > 0)
            axes(motor_init_axes); 
            motor_units = zeros(2,2);      
            area_ind = NodInitE_ind; 
            fetch_area_act;     
            motor_units(1,1) = activs;
            area_ind = ShakeInitE_ind; 
            fetch_area_act;       
            motor_units(2,1) = activs;   
            area_ind = NodInitI_ind; 
            fetch_area_act;     
            motor_units(1,2) = activs;
            area_ind = ShakeInitI_ind; 
            fetch_area_act;       
            motor_units(2,2) = activs;             
            set(motor_init_im,'CData',motor_units);   
        end;

        % Draw babble units.
        plot_id = 9;
        if (subplot_inds(plot_id,3) > 0)
            axes(babble_units_axes);
            area_ind = Babble_ind; 
            fetch_area_act; 
            set(babble_units_im,'CData',activs);
        end;

        % Draw babble drive level.
        plot_id = 10;
        if (subplot_inds(plot_id,3) > 0)
            axes(babble_drive_level_axes);
            area_ind = BabbleDriveUnit_ind; 
            fetch_area_act; 
            set(babble_drive_level_im,'CData',activs);
        end;   
    end;    
end;

% Save simulation information for future viewing.
stim_on_sv(iter) = stim_on;
stim_x_sv(iter) = stim_x;
stim_y_sv(iter) = stim_y;
stim_red_sv(iter) = stim_red;
stim_green_sv(iter) = stim_green;
stim_blue_sv(iter) = stim_blue;
tones_on_sv(:,iter) = tones_on;
reward_on_sv(iter) = reward_on;
reward_trigger_sv(iter) = reward_trigger;
reward_delivered_sv(iter) = reward_delivered;
punish_on_sv(iter) = punish_on;
punish_trigger_sv(iter) = punish_trigger;
punish_delivered_sv(iter) = punish_delivered;
novelty_on_sv(iter) = novelty_on;
camera_x_sv(iter) = camera_x;
camera_y_sv(iter) = camera_y;
mem_color_map_sv(:,iter) = mem_color_map;
mem_tones_map_sv(:,iter) = mem_tones_map;
track_execute_sv(iter) = track_execute;
nod_execute_sv(iter) = nod_execute;
shake_execute_sv(iter) = shake_execute;

end;  % iteration

% Show the trial type counts and performance measures.
target_count
distractor_count
delay_count
reward_count
punish_count
target_nod
target_shake
target_x_nod
target_nod_shake
target_miss
distractor_nod
distractor_shake
distractor_x_shake
distractor_nod_shake
distractor_miss
delay_nod
delay_shake
delay_nod_shake
blind_nod
blind_shake

target_nod_po = 1;  % 'po' = payoff
target_shake_po = -1;
target_x_nod_po = -0.5;
target_miss_po = -1;
target_nod_shake_po = -1;
distractor_nod_po = -1;
distractor_shake_po = 1;
distractor_x_shake_po = -0.5;
distractor_miss_po = -1;
distractor_nod_shake_po = -1;
delay_nod_po = -1;
delay_shake_po = -1;
delay_nod_shake_po = -1;
blind_nod_po = -1;
blind_shake_po = -1;

% Best measure here would be 2.00.
% For no response system, performance is -2.00.
% For red_select response when blue_select is ideal, result is about -1.5.
perf_measure = 0;
% if (target_count > 0)
%     perf_measure = perf_measure + ...
%         target_nod * target_nod_po / target_count + ...
%         target_shake * target_shake_po / target_count + ...
%         target_x_nod * target_x_nod_po / target_count + ...
%         target_miss * target_miss_po / target_count + ...  
%         target_nod_shake * target_nod_shake_po / target_count + ...  
%         blind_nod * blind_nod_po / target_count;
% end;
% if (distractor_count > 0)
%      perf_measure = perf_measure + ...
%         distractor_nod * distractor_nod_po / distractor_count + ...
%         distractor_shake * distractor_shake_po / distractor_count + ...
%         distractor_x_shake * distractor_x_shake_po / distractor_count + ...
%         distractor_miss * distractor_miss_po / distractor_count + ... 
%         distractor_nod_shake * distractor_nod_shake_po / distractor_count + ...  
%         blind_shake * blind_shake_po / distractor_count;    
% end;
% if (delay_count > 0)
%      perf_measure = perf_measure + ... 
%         delay_nod * delay_nod_po / delay_count + ...
%         delay_shake * delay_shake_po / delay_count + ...
%         delay_nod_shake * delay_nod_shake_po / delay_count;     
% end;
% perf_measure

% Show the percentage of trials that were rewarded.
reward_pct = reward_count * 100 / (target_count + distractor_count);
reward_pct

if (brain_type == brain_cog_model)
    babble_count
    nod_babble_count
    shake_babble_count
    nod_shake_babble_count
end;

% Save the simulation output.
save(sim_out_fname,'num_iters','iter_duration','scene_width', ...
    'scene_height','stim_shape','camera_width','camera_height', ...
    'fovea_width','fovea_height','stim_on_sv','stim_x_sv','stim_y_sv', ...
    'stim_red_sv','stim_green_sv','stim_blue_sv','tones_on_sv', ...
    'reward_on_sv','reward_trigger_sv','reward_delivered_sv', ...
    'punish_on_sv','punish_trigger_sv','punish_delivered_sv', ...    
    'novelty_on_sv', 'brain_type','camera_x_sv','camera_y_sv', ...
    'mem_color_map_sv','mem_tones_map_sv','track_execute_sv', ...
    'nod_execute_sv','shake_execute_sv','target_count', ...
    'distractor_count','delay_count','reward_count','punish_count', ...
    'target_nod','target_shake','target_x_nod','target_nod_shake', ...
    'target_miss','distractor_nod','distractor_shake', ...
    'distractor_x_shake','distractor_nod_shake','distractor_miss', ...
    'delay_nod','delay_shake','delay_nod_shake', ...
    'blind_nod','blind_shake','perf_measure','reward_pct');

% If we're running a cognitive (neural) model...
if (brain_type == brain_cog_model)
    % Save the brain output
    save(cogmodel_out_fname,'output','mdc','modCW','elig_trace', ...
        'base_threshs','thresh_adjs','num_iters','babble_count', ...
        'nod_babble_count','shake_babble_count');

    % If we're doing learning and are supposed to, save new weights for the 
    % model.
    if ((~isempty(mdc)) && (learning_on == 1) && (save_trained_wts == 1))
        sfname = strcat(train_model_fname,'.mat');
        tfname = strcat(prev_train_model_fname,'.mat');
        cp_err = copyfile(sfname,tfname);
        if (lock_model == 0)
            save(train_model_fname,'Model_name','Num_areas','Num_conns', ...
                'Area_name','Area_num','Cell_type','Area_xy','Conn_name', ...
                'Conn_fromto','Chan_type','Conn_kind','Conn_xy','Conn_prob', ...
                'Weig_mean','Weig_sigm','Conn_scle','CA','CW','CR','CH', ...
                'CT','NC','NU','unit_offset','novcount_color_map', ...
                'novcount_tones','frustration_level');
        else
            disp('Warning... New weights not saved because model is locked.');
        end;
    end;
end;